import math
from math import radians, degrees
from random import randint

import pyglet
import pymunk as pm
from pyglet.window import key
from pymunk.vec2d import Vec2d

from res import Images, Sound


import sprite
import helper

class LawnMower(sprite.PmSprite):
    
    def __init__(self, img, window, space, cats, *args, **kwargs):
        sprite.PmSprite.__init__(self, img, window, space, *args, **kwargs)
        
        self.state = "alive"
        self.cats = cats
        self.cats_caught = []
        self.chain_dist = 20
        
        verts = helper.verts_img(self.image)
        self.mass = 1
        self.inertia = pm.moment_for_poly(self.mass, verts, Vec2d(0,0))
        self.move_speed = 13
        self.steer_speed = 4
        
        self.body = pm.Body(self.mass, self.inertia)
        self.pm_shape = pm.Poly(self.body, verts, Vec2d(0,0))
        self.pm_shape.friction = 0.3
        
    def update(self,dt):
        if self.state == "alive":
            if self.ran_over_own_cat():
                self.state = "game_over"
                game_over = Sound.game_over
                game_over.play()
                self.window.state.is_game_over = True
            self.move(dt)
            cat_caught = self.is_cat_caught()
            if cat_caught:
                #print 'yes'
                self.attach_cat(cat_caught)
        
        if self.state == "game_over":
            pass               
        
        #self.wrap()
        self.position = self.body.position
        self.rotation = degrees(self.body.angle)
        
    def move(self,dt):
        if not self.window.keys[key.DOWN]:
            f = self.body._bodycontents.rot * (self.move_speed + len(self.cats_caught))
            self.body.apply_impulse(Vec2d(f.x, -f.y) , (0,0))
        if self.window.keys[key.DOWN]:
            f = -(self.body._bodycontents.rot * self.move_speed)
            self.body.apply_impulse(Vec2d(f.x, -f.y) , (0,0))
            
        if self.window.keys[key.RIGHT]:
            self.body.angle += radians(self.steer_speed)
        elif self.window.keys[key.LEFT]:
            self.body.angle -= radians(self.steer_speed)
            
    def is_cat_caught(self):
        for arb in self.space.arbiters:
            if (arb.a == self.pm_shape) or (arb.b == self.pm_shape):
                if arb.a == self.pm_shape:
                    for c in self.cats:
                        if c.pm_shape == arb.b:
                            if c.state != "caught":
                                return c
                elif arb.b == self.pm_shape:
                    for c in self.cats:
                        if c.pm_shape == arb.a:
                            if c.state != "caught":
                                return c
        return False
    
    def ran_over_own_cat(self):
        for arb in self.space.arbiters:
            if (arb.a == self.pm_shape) or (arb.b == self.pm_shape):
                if arb.a == self.pm_shape:
                    for c in self.cats_caught:
                        if c.pm_shape == arb.b:
                            return True
                elif arb.b == self.pm_shape:
                    for c in self.cats_caught:
                        if c.pm_shape == arb.a:
                            return True
        return False
    
    def attach_cat(self,cat):
        if self.cats_caught == []:    
            rot = self.body._bodycontents.rot
            offset = Vec2d(rot.x,-rot.y) * (self.chain_dist+10)
            pos = self.body.position -  offset
            cat.body.position = pos
        
            cat.chain_to(self)
            
            self.cats_caught.append(cat)
            
        elif self.cats_caught:           
            last_cat = self.cats_caught[-1]
                 
            rot = last_cat.body._bodycontents.rot
            offset = Vec2d(rot.x,-rot.y) * self.chain_dist
            pos = last_cat.body.position -  offset
            cat.body.position = pos
            
            cat.chain_to(last_cat)
            
            self.cats_caught.append(cat)
        
        pickup = Sound.pickup
        pickup.play()
            
    def get_random_target(self,x1=50,y1=50,x2=640,y2=480):
       # width, height = self.window.get_size()
        x = randint(abs(x1),abs(x2))
        y = randint(abs(y1),abs(y2))
        if x > 640: x = 639
        if x < 0: x = 1
        if y > 480: y = 379
        if y < 0 : y =1
        return Vec2d(x,y)
   

