import math
from math import atan2, radians, degrees 

import pyglet
import pymunk as pm



class SpriteManager(object):
    
    def __init__(self,space=None):
        self.sprites = []
        self.space = space
        self.batch = pyglet.graphics.Batch()
        
    def update(self,dt):
        for s in self.sprites:
            s.update(dt)
    
    def add(self,s):
        self.sprites.append(s)
        s.batch = self.batch
        
    def remove(self,s):
        s.batch = None
        self.sprites.remove(s)
        del s
        
    
        

class BaseSprite(pyglet.sprite.Sprite):
    "My Sprite"
    
    def __init__(self, img, window,*args, **kwargs):
        pyglet.sprite.Sprite.__init__(self, img, *args, **kwargs)
        self.window = window
        self.scale = 0.6
        
    def update(self, dt):
        pass
        
    def collide(self, other):
        if other.x >= self.x \
        and other.x <= self.x + self.width \
        and other.y >= self.y - self.height \
        and other.y <= self.y:
            return True
    
    
        
    
class PmSprite(BaseSprite):
    "Sprite that supports pymunk stuff"
    def __init__(self, img, window, space, *args, **kwargs):
        BaseSprite.__init__(self, img, window, *args, **kwargs)
        self.space = space
        self.body = pm.Body(1,1)
        
    def wrap(self):
        pos = self.body.position
        width, height = self.window.get_size()
        if pos.x > width:   pos.x = 0
        if pos.x < 0:       pos.x = width
        if pos.y > height:  pos.y = 0
        if pos.y < 0:       pos.y = height
        self.body.position = pos
    

    
        
         