import pyglet
import random
from random import randint
import pymunk as pm
from pyglet.gl import *
from pyglet.window import key 
from pymunk.vec2d import Vec2d

import sprite
from res import Images, Sound
from lawnmower import LawnMower
from cat import Cat
from button import Button


class BaseState(object):
    "Base for all state"
    
    def __init__(self,window):
        self.window = window
        self.batches = []
        self.default_batch = pyglet.graphics.Batch()
        self.batches.append(self.default_batch)
        glClearColor(0.0,1.0,0.4,1.0)

    def on_draw(self):
        glClear(GL_COLOR_BUFFER_BIT)
        for batch in self.batches:
            batch.draw()
    
    def update(self,dt):
        pass
    
    
    
    
class MenuState(BaseState):
    "Start here please"
    
    def __init__(self,window):
        BaseState.__init__(self, window)
        self.title= pyglet.text.Label("Cat-Mower game")
        self.title.x, self.title.y = 340,390
        self.title.batch = self.default_batch
        self.title.color = (0,0,0,255)
        self.title.anchor_x = self.title.anchor_y = "center"
        self.title.font_name = "Verdana"
        self.title.font_size = 30
        
        self.banner = Images.banner
        
        self.buttons = {
    "new_game" : Button("New Game", self.default_batch, self.window, x=200,y=300),
    "quit" : Button("Quit", self.default_batch, self.window, x=200,y=200)}
        
    def update(self,dt):
        
        for button in self.buttons.values():
            button.update(dt)
            
        if self.buttons["new_game"].clicked:
            sound = Sound.pickup
            sound.play()
            self.window.set_state(GameState(self.window))
        if self.buttons["quit"].clicked:
            pyglet.app.exit()
            
    def on_draw(self):
        
        glClear(GL_COLOR_BUFFER_BIT)
        self.banner.blit(320,240)
        for batch in self.batches:
            batch.draw()
    
    
    def on_mouse_press(self, x, y, button, modifiers):
        for button in self.buttons.values():
            button.on_mouse_press(x, y, button, modifiers)
        
        
        
class GameState(BaseState):
    "Gameplay here"
    
    def __init__(self,window,level=1):
        BaseState.__init__(self, window)
        
        self.sprite_mgr = None
        self.cats = []
        self.lawn_mower = None
        self.space = None
        
        self.is_game_over = False
        
        self.caught_text = pyglet.text.Label()
        self.bg = Images.bg
        
        
        self.game_over = pyglet.text.Label()
        self.game_over.text = "Game Over! - You ran over your cat(s)"
        self.game_over.x, self.game_over.y = 320,240
        self.game_over.anchor_x = "center"
        self.game_over.anchor_y = "center"
        self.game_over.font_size = 20
        self.game_over.color = (0,0,0,255)
        
        self.level = level
        
        self.start_level()
        
    def update(self,dt):
        if not self.is_game_over:
            self.sprite_mgr.update(dt)
            self.caught_text.text = \
                "Cats caught and chained: " + str(len(self.lawn_mower.cats_caught))
            self.space.step(dt)
        
        if self.some_cats_left(self.level+2):
            self.level += 1
            for c in range(int(self.level*1.5)):
                c = Cat(Images.cat_explore, self.window, self.space, self.lawn_mower)
                c.body.position = c.get_random_target()
                c.set_new_target()
                self.cats.append(c)
                self.sprite_mgr.add(c)
                self.space.add(c.body, c.pm_shape)
            
    def on_draw(self):
        glClear(GL_COLOR_BUFFER_BIT)
        self.bg.blit(320,240)
        for batch in self.batches:
            batch.draw()
        self.caught_text.draw()
        if self.is_game_over:
            self.game_over.draw()
            
    def on_key_press(self, symbol, modifiers):
        if self.is_game_over:
            #self.end_level()
            self.window.set_state(MenuState(self.window))
        elif symbol == key.ESCAPE:
            self.window.set_state(MenuState(self.window))
            return True
            
    
    def start_level(self):
        self.sprite_mgr = sprite.SpriteManager()
        self.batches.append(self.sprite_mgr.batch)
        
        
        self.space = pm.Space()
        self.space.damping = 0.05
        self.cats = []
        
        self.wall_body = pm.Body(pm.inf,pm.inf)
        self.wall_body.position = Vec2d(320,240)
        self.wall_bottom = pm.Segment(self.wall_body,Vec2d(-320,-240), Vec2d(320,-240), 3)
        self.wall_top    = pm.Segment(self.wall_body,Vec2d(-320,240),  Vec2d(320,240), 3)
        self.wall_left   = pm.Segment(self.wall_body,Vec2d(-320,240),  Vec2d(-320,-240), 3)
        self.wall_right  = pm.Segment(self.wall_body,Vec2d(320,240),   Vec2d(320,-240), 3)
        self.space.add(self.wall_body,self.wall_bottom,
                        self.wall_top, self.wall_left, self.wall_right)
        
        self.lawn_mower = LawnMower(Images.lawn_mower, self.window, self.space, self.cats)
        self.lawn_mower.batch = self.default_batch
        self.lawn_mower.body.position = self.lawn_mower.get_random_target()
        self.lawn_mower.body.angle = randint(0,360)
        self.sprite_mgr.add(self.lawn_mower)
        self.space.add(self.lawn_mower.body, self.lawn_mower.pm_shape)
        
        for c in range(self.level*5):
            c = Cat(Images.cat_explore, self.window, self.space, self.lawn_mower)
            c.body.position = c.get_random_target()
            c.set_new_target()
            self.cats.append(c)
            self.sprite_mgr.add(c)
            self.space.add(c.body, c.pm_shape)
            
        self.caught_text = pyglet.text.Label()
        self.caught_text.text = \
            "Cats caught and chained: " + str(len(self.lawn_mower.cats_caught))
        self.caught_text.x, self.caught_text.y = 30,30
        self.caught_text.font_size = 10
        self.caught_text.color = (0,0,0,255)
            
    def end_level(self):
        self.space.remove(self.lawn_mower.body, self.lawn_mower.pm_shape)
        self.sprite_mgr.remove(self.lawn_mower)
        
        for c in self.cats:
            self.cats.remove(c)
            self.space.remove(c.body, c.pm_shape, c.chain)
            self.sprite_mgr.remove(c)
        
        self.batches.remove(self.sprite_mgr.batch)    
        
        del self.cats
        del self.space
        del self.sprite_mgr
         
    def is_all_cats_caught(self):
        if len(self.lawn_mower.cats_caught) == len(self.cats):
            return True
        else:
            return False
        
    def some_cats_left(self,num):
        if len(self.lawn_mower.cats_caught) >= (len(self.cats)-num):
            return True
        else:
            return False
        
    def draw_chains(self):
        for i in range(len(self.lawn_mower.cats_caught)):
            cat = self.lawn_mower.cats_caught[i-1]
            #if cat == self.lawn_mower.cats_caught[-1]:
            #    return
            last_cat = self.lawn_mower.cats_caught[i]
            cat_pos = cat.body.position
            last_cat_pos = last_cat.body.position
            glColor3f(0.0,0.0,0.0)
            glBegin(GL_LINES)
            glVertex2f(cat_pos.x, cat_pos.y)
            glVertex2f(last_cat_pos.x, last_cat_pos.y)
            glEnd()
        